/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - many resource settings were moved to the app-defaults file
   - a status bar is displayed below the text widget
   - the order of menu entries in the File and Edit menu was
     changed according to the motif style guide

							06.09.1997
 */

#include <stdio.h>
#include <string.h>

#include <Xm/Xm.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Separator.h>
#include <Xm/LabelG.h>

#include "sys_consts.h"
#include "globals.h"
#include "widgets.h"
#include "callbacks.h"

/***  GLOBAL VARIABLES  ***/
Widget   text_canvas;
Widget   statusBar;
Widget   undoMenuPB, redoMenuPB, replaceMenuPB;
Widget   jumpMenuText;
Widget   searchMenuText, replaceMenuText;


/***  LOCAL VARIABLES  ***/
static Widget  toolBar;
static Widget  mainFO;

/***  PROCEDURE DECLARATIONS  ***/
static void gui_createToolBar(Widget parent);


/* CREATE INTERFACE */
void     gui_createWidgets(Widget toplevel)
{
   Atom     WM_DEL;
   Widget   s_frame;

/* react on the close command from the window manager menu */
   WM_DEL = XmInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(toplevel, WM_DEL, gui_quitCB, (XtPointer)NULL);

/* MAIN FORM WIDGET */
   mainFO = XtVaCreateWidget("mainFO", xmFormWidgetClass, toplevel, NULL);

/* TOOL BAR WIDGET */
   toolBar = XtVaCreateWidget("toolBar", xmRowColumnWidgetClass, mainFO,
			XmNtopAttachment,   XmATTACH_FORM,
			XmNleftAttachment,  XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			XmNresizeHeight,    True,
			XmNspacing,         3,
			XmNadjustLast,      True,
			NULL);
   gui_createToolBar(toolBar);
   XtManageChild(toolBar);

/* STATUS BAR */
   s_frame = XtVaCreateWidget("statusFrame", xmFrameWidgetClass, mainFO,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment,   XmATTACH_FORM,
			XmNrightAttachment,  XmATTACH_FORM,
			NULL);
   statusBar = XtVaCreateManagedWidget("statusBar", xmLabelGadgetClass, s_frame, NULL);
   XtManageChild(s_frame);

/* THE EDITOR ITSELF */
   create_text_canvas();

   XtManageChild(mainFO);
}


void create_text_canvas(void)
{
   Arg      args[20];
   int      n;
   static XmTextScanType sarray[] = {
      XmSELECT_POSITION, XmSELECT_WORD,
      XmSELECT_LINE,     XmSELECT_PARAGRAPH,
      XmSELECT_ALL
   };

/* CREATE TEXT CANVAS */
   n = 0;
   XtSetArg(args[n], XmNeditMode,            XmMULTI_LINE_EDIT); n++;
   XtSetArg(args[n], XmNselectionArrayCount, 5);                 n++;
   XtSetArg(args[n], XmNselectionArray,      sarray);            n++;
   XtSetArg(args[n], XmNtopAttachment,       XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNtopWidget,           toolBar);           n++;
   XtSetArg(args[n], XmNbottomAttachment,    XmATTACH_WIDGET);   n++;
   XtSetArg(args[n], XmNbottomWidget,        statusBar);         n++;
   XtSetArg(args[n], XmNleftAttachment,      XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNrightAttachment,     XmATTACH_FORM);     n++;
   XtSetArg(args[n], XmNwordWrap,            WordWrap);          n++;
#ifdef TOGGLE_SCROLLBAR
   XtSetArg(args[n], XmNscrollHorizontal,    !WordWrap);         n++;
#else
   XtSetArg(args[n], XmNscrollHorizontal,    False);             n++;
#endif

   text_canvas = XmCreateScrolledText(mainFO, "text_canvas", args, n);
   XtAddCallback(text_canvas, XmNmodifyVerifyCallback, gui_keypressCB, NULL);
   XtManageChild(text_canvas);
}

static void installHelpCBs(Widget w, XtCallbackProc p, XtPointer d, int id)
{
   XtAddCallback(w, XmNactivateCallback, p, d);
   XtAddCallback(w, XmNarmCallback, gui_contextSensitiveHelpCB, (XtPointer)id);
   XtAddCallback(w, XmNdisarmCallback, gui_undoContextSensitiveHelpCB, (XtPointer)id);
}

static void gui_createToolBar(Widget parent)
{
   Widget   menuBar;
   Widget   secondBar, secondBarFrame, helpPullDown, searchPullDown;
   Widget   optionPullDown, filePullDown, editPullDown;
   Widget   menuPB;
   Widget   lbl;

/***********************************************************************************/
/******************************TOOL BAR ********************************************/
/***********************************************************************************/

/* MENU BAR */
   menuBar = XmCreateMenuBar(parent, "menuBar", NULL, 0);

/********************************************************************************/
/******   FILE PULLDOWN *********************************************************/
/********************************************************************************/
/* FILE PULLDOWN */
   filePullDown = XmCreatePulldownMenu(menuBar, "filePullDown", NULL, 0);

   XtVaCreateManagedWidget("fileB", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, filePullDown,
			   NULL);

/* NEW BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("newB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_newCB, NULL, NEW);

/* OPEN BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("openB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_browseCB, (XtPointer)OPENIT, OPEN);

/* INSERT FILE BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("insB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_browseCB, (XtPointer)INSERTIT, INSERT);

   XtVaCreateManagedWidget("Separator", xmSeparatorWidgetClass, filePullDown, NULL);

/* SAVE BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("saveB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_saveCB, NULL, SAVE);

/* SAVE AS BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("saveAsB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_saveAsCB, NULL, SAVEAS);

/* SAVE SELECTION BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("saveSelB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_saveSelectionCB, NULL, SAVESELECTION);

   XtVaCreateManagedWidget("Separator", xmSeparatorWidgetClass, filePullDown, NULL);

/* PRINT BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("printB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_printCB, NULL, PRINT);

/* PRINT SELECTION BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("printSelB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_printSelectionCB, NULL, PRINTSELECTION);

   XtVaCreateManagedWidget("Separator", xmSeparatorWidgetClass, filePullDown, NULL);

/* QUIT BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("quitB", xmPushButtonGadgetClass, filePullDown, NULL);
   installHelpCBs(menuPB, gui_quitCB, NULL, QUIT);

/********************************************************************************/
/******   EDIT PULLDOWN *********************************************************/
/********************************************************************************/
/* EDIT PULLDOWN */
   editPullDown = XmCreatePulldownMenu(menuBar, "editPullDown", NULL, 0);

   XtVaCreateManagedWidget("editB", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, editPullDown,
			   NULL);

/* UNDO BUTTON WIDGET */
   undoMenuPB = XtVaCreateManagedWidget("undoB", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(undoMenuPB, gui_undoCB, (XtPointer)UNDO, UNDO);

/* REDO BUTTON WIDGET */
   redoMenuPB = XtVaCreateManagedWidget("redoB", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(redoMenuPB, gui_undoCB, (XtPointer)REDO, REDO);

   XtVaCreateManagedWidget("Separator", xmSeparatorWidgetClass, editPullDown, NULL);

/* COPY BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("copyB", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, cut_copyCB, (XtPointer)1, COPY);

/* CUT BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("cutB", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, cut_copyCB, (XtPointer)0, CUT);

/* PASTE BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("pasteB", xmPushButtonGadgetClass, editPullDown, NULL);
   installHelpCBs(menuPB, gui_pasteCB, NULL, PASTE);

/********************************************************************************/
/******   SEARCH PULLDOWN *******************************************************/
/********************************************************************************/
/* CREATE SEARCH PULL DOWN */
   searchPullDown = XmCreatePulldownMenu(menuBar, "searchPullDown", NULL, 0);

   XtVaCreateManagedWidget("searchB", xmCascadeButtonWidgetClass, menuBar,
			   XmNsubMenuId, searchPullDown,
			   NULL);

/* SEARCH BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("srchForB", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_searchReplaceCB, (XtPointer)SEARCH, SEARCH);

/* REPLACE BUTTON WIDGET */
   replaceMenuPB = XtVaCreateManagedWidget("replaceB", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(replaceMenuPB, gui_searchReplaceCB, (XtPointer)REPLACE, REPLACE);
   XtSetSensitive(replaceMenuPB, False);

/* REPLACE ALL BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("replAllB", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_searchReplaceCB, (XtPointer)REPLACEALL, REPLACEALL);

/* TOGGLE SELECTION BUTTON */
   menuPB = XtVaCreateManagedWidget("inSelB", xmToggleButtonWidgetClass, searchPullDown,
			XmNset, False,
			NULL);
   XtAddCallback(menuPB, XmNvalueChangedCallback, searchSelectionOnlyToggleCB, NULL);

/* MATCH BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("matchB", xmPushButtonGadgetClass, searchPullDown, NULL);
   installHelpCBs(menuPB, gui_matchCB, NULL, MATCH);

/********************************************************************************/
/******   OPTIONS PULLDOWN *******************************************************/
/********************************************************************************/
/* CREATE OPTION PULL DOWN */
   optionPullDown = XmCreatePulldownMenu(menuBar, "optionPullDown", NULL, 0);
   XtVaCreateManagedWidget("optionB", xmCascadeButtonWidgetClass, menuBar,
			XmNsubMenuId, optionPullDown,
			NULL);

/* setUp BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("setupB", xmPushButtonGadgetClass, optionPullDown, NULL);
   installHelpCBs(menuPB, gui_setUpCB, NULL, SETUP);

/********************************************************************************/
/******   HELP PULLDOWN *******************************************************/
/********************************************************************************/
/* CREATE HELP PULL DOWN */
   helpPullDown = XmCreatePulldownMenu(menuBar, "helpPullDown", NULL, 0);
   menuPB =XtVaCreateManagedWidget("helpM", xmCascadeButtonWidgetClass, menuBar,
			XmNsubMenuId, helpPullDown,
			NULL);
   XtVaSetValues(menuBar, XmNmenuHelpWidget,menuPB, NULL);

/* HELP BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("helpB", xmPushButtonGadgetClass, helpPullDown, NULL);
   installHelpCBs(menuPB, gui_helpCB, (XtPointer)HELPCALL, HELPB);

   menuPB = XtVaCreateManagedWidget("aboutB", xmPushButtonGadgetClass, helpPullDown, NULL);
   XtAddCallback(menuPB, XmNactivateCallback, gui_helpCB, (XtPointer)HELPABOUT);

   XtManageChild(menuBar);

/********************************************************************************/
/******************** BAR FOR TEXT WIDGETS **************************************/
/********************************************************************************/

/* CREATE FRAME FOR SECOND BAR AREA */
   secondBarFrame = XtVaCreateWidget("secondBarFrame", xmFrameWidgetClass, toolBar, NULL);

/* BUTTON BAR WIDGET */
   secondBar = XtVaCreateWidget("secondBar", xmFormWidgetClass, secondBarFrame, NULL);

/* JUMP BUTTON WIDGET */
   menuPB = XtVaCreateManagedWidget("jumpB", xmPushButtonWidgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_FORM,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   installHelpCBs(menuPB, gui_jumpCB, NULL, JUMP);

/* JUMP MenuText Widget */
   jumpMenuText = XtVaCreateManagedWidget("jumpMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        8,
				XmNcolumns,          8,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       menuPB,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(jumpMenuText, XmNactivateCallback, gui_jumpCB, NULL);
   XtAddCallback(jumpMenuText, XmNfocusCallback, gui_contextSensitiveHelpCB, (XtPointer)JUMPTEXT);
   XtAddCallback(jumpMenuText, XmNlosingFocusCallback, gui_undoContextSensitiveHelpCB, (XtPointer)JUMPTEXT);

/* SEARCH LABEL WIDGET */
   lbl = XtVaCreateManagedWidget("searchB", xmLabelGadgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       jumpMenuText,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);

/* SEARCH MenuText Widget */
   searchMenuText = XtVaCreateManagedWidget("searchMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        100,
				XmNcolumns,          17,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       lbl,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(searchMenuText, XmNactivateCallback, gui_searchReplaceCB, (XtPointer)SEARCH);
   XtAddCallback(searchMenuText, XmNfocusCallback, gui_contextSensitiveHelpCB, (XtPointer)SEARCHTEXT);
   XtAddCallback(searchMenuText, XmNlosingFocusCallback, gui_undoContextSensitiveHelpCB, (XtPointer)SEARCHTEXT);

/* REPLACE LABEL WIDGET */
   lbl = XtVaCreateManagedWidget("replaceB", xmLabelGadgetClass, secondBar,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       searchMenuText,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);

/* REPLACE MenuText Widget */
   replaceMenuText = XtVaCreateManagedWidget("replaceMenuText", xmTextFieldWidgetClass, secondBar,
				XmNmaxLength,        100,
				XmNcolumns,          17,
				XmNtopAttachment,    XmATTACH_FORM,
				XmNleftAttachment,   XmATTACH_WIDGET,
				XmNleftWidget,       lbl,
				XmNbottomAttachment, XmATTACH_FORM,
				NULL);
   XtAddCallback(replaceMenuText, XmNfocusCallback, gui_contextSensitiveHelpCB, (XtPointer)REPLACETEXT);
   XtAddCallback(replaceMenuText, XmNlosingFocusCallback, gui_undoContextSensitiveHelpCB, (XtPointer)REPLACETEXT);

   XtManageChild(secondBar);
   XtManageChild(secondBarFrame);
}
